from CvPythonExtensions import *
from StateRules import *
from StateXMLParser import *
	
import SdToolKitCustom as SDTK

# globals
game = CyGame()

class StateModule :
	### all rules
	lpAvailableStateRules = []
	
	def __init__( self, fileNames ) :
		self.lpAvailableStateRules = readStateRules( fileNames )
		# debug
		print "HOMELAND: state rules read:"
		for pStateRule in self.lpAvailableStateRules :
			pStateRule.prnt()
		if( not SDTK.sdObjectExists( "StateManager", game ) ) :
			print "StateManager: creating SD Objects"
			SDTK.sdObjectInit( "StateManager", game, {} )
			SDTK.sdObjectSetVal( "StateManager", game, "dipRulesForPlayer", {} )
		print SDTK.sdObjectGetVal( "StateManager", game, "dipRulesForPlayer" )
	
	def getStateRuleForPlayer( self, sProvince, iPlayer ) :
		self.updateStateRule( sProvince, iPlayer )
		dipRulesForPlayer = SDTK.sdObjectGetVal( "StateManager", game, "dipRulesForPlayer" )
		return dipRulesForPlayer[iPlayer]
	
	def getStateRuleForCities( self, sProvince, lpCities ) :
		# TODO: update StateRules when civ is created? -> No, Civ updates when asking for name
		return self.chooseBestAvailableStateRuleForCities( sProvince, lpCities )
	
	def updateStateRule( self, sProvince, iPlayer ):
		#print "updateStateRule( %s, %i )"%( sProvince, iPlayer )
		dipRulesForPlayer = SDTK.sdObjectGetVal( "StateManager", game, "dipRulesForPlayer" )
		
		iEra, iStateReligion, diiReligionInfluence, iCiv = self.getStateRuleParamsForPlayer( iPlayer )
		
		# only update if score of old stateRule is 0 or not there
		if( not( iPlayer in dipRulesForPlayer ) or ( dipRulesForPlayer[iPlayer] == None ) or ( dipRulesForPlayer[iPlayer].getPoints( sProvince, iEra, diiReligionInfluence, iStateReligion, iCiv ) == 0 ) ) :
			dipRulesForPlayer[iPlayer] = self.chooseBestAvailableStateRuleForPlayer( sProvince, iPlayer )
		SDTK.sdObjectSetVal( "StateManager", game, "dipRulesForPlayer", dipRulesForPlayer )
	
	def getStateRuleParamsForPlayer( self, iPlayer ) :
		iEra = gc.getPlayer( iPlayer ).getCurrentEra()
		iStateReligion = gc.getPlayer( iPlayer ).getStateReligion()
		
		diiReligionInfluence = {}
		
		for iCityIndex in xrange( gc.getPlayer( iPlayer ).getNumCities() ) :
			pCity = gc.getPlayer( iPlayer ).getCity( iCityIndex )
			liCityReligions = []
			for iRel in xrange( gc.getNumReligionInfos() ) :
				if( pCity.isHasReligion( iRel ) ) :
					print "HOMELAND: has religion:"
					print iRel
					liCityReligions.append( iRel )
			#debug
			print "HOMELAND: liCityReligions:"
			print liCityReligions
			for iRel in xrange( gc.getNumReligionInfos() ) :
				diiReligionInfluence[iRel] = 0
				if( iRel in liCityReligions and iRel in diiReligionInfluence ) :
					diiReligionInfluence[iRel] += 1 / len( liCityReligions ) / gc.getPlayer( iPlayer ).getNumCities()
		#debug
		print "HOMELAND: diiReligionInfluence:"
		print diiReligionInfluence
		
		iCiv = gc.getPlayer( iPlayer ).getCivilizationType()
		
		return [iEra, iStateReligion, diiReligionInfluence, iCiv]
	
	def chooseBestAvailableStateRuleForPlayer( self, sProvince, iPlayer ) :
		print "HOMELAND: Searching for rules in Province %s (Player)"%( sProvince )
		
		dipRulesForPlayer = SDTK.sdObjectGetVal( "StateManager", game, "dipRulesForPlayer" )
		
		iEra, iStateReligion, diiReligionInfluence, iCiv = self.getStateRuleParamsForPlayer( iPlayer )
		
		maxPoints = 0
		pCurrentRule = None
		
		for pStateRule in self.lpAvailableStateRules :
			print "HOMELAND: Found rule in Province %s"%( sProvince )
			iCurrentPoints = pStateRule.getPoints( sProvince, iEra, diiReligionInfluence, iStateReligion, iCiv )
			if iCurrentPoints > maxPoints :
				#check if that or a similar StateRule is used
				# TODO: What if Civ is already used?
				bProvinceUnused = True
				for iLoopPlayer, pRule in dipRulesForPlayer.iteritems() :
					if not ( pRule == None or iPlayer == iLoopPlayer ):
						if( gc.getPlayer( iLoopPlayer ).isAlive() and ( pStateRule == pRule or pStateRule.sDesc == pRule.sDesc or pStateRule.sShortDesc == pRule.sShortDesc or pStateRule.sAdjective == pRule.sAdjective  ) ):
							bProvinceUnused = False
							break
				if( bProvinceUnused ) :
					maxPoints = iCurrentPoints
					pCurrentRule = pStateRule
		return pCurrentRule
	
	def chooseBestAvailableStateRuleForCities( self, sProvince, lpCities ) :
		print "HOMELAND: Searching for rules in Province %s (Cities)"%( sProvince )
		
		dipRulesForPlayer = SDTK.sdObjectGetVal( "StateManager", game, "dipRulesForPlayer" )
		
		# Era
		iEra = -1
		## Chooses era of most players
		diiEraPoints = {}
		for iPlayer in xrange( gc.getMAX_PLAYERS() ) :
			if not gc.getPlayer( iPlayer ).getCurrentEra() in diiEraPoints :
				diiEraPoints[gc.getPlayer( iPlayer ).getCurrentEra()] = 0
			diiEraPoints[gc.getPlayer( iPlayer ).getCurrentEra()] += 1
		iMaxPoints = 0
		
		for (iLoopEra, iPoints) in diiEraPoints.iteritems() :
			if( iPoints > iMaxPoints ) :
				iMaxPoints = iPoints
				iEra = iLoopEra
		
		# StateReligion
		iStateReligion = -1
		
		# ReligionInfluence
		diiReligionInfluence = {}
		
		for pCity in lpCities :
			liCityReligions = []
			for iRel in xrange( gc.getNumReligionInfos() ) :
				if( pCity.isHasReligion( iRel ) ) :
					print "HOMELAND: has religion:"
					print iRel
					liCityReligions.append( iRel )
			#debug
			print "HOMELAND: liCityReligions:"
			print liCityReligions
			for iRel in xrange( gc.getNumReligionInfos() ) :
				diiReligionInfluence[iRel] = 0
				if( iRel in liCityReligions and iRel in diiReligionInfluence ) :
					diiReligionInfluence[iRel] += 1 / len( liCityReligions ) / len( lpCities )
		#debug
		print "HOMELAND: diiReligionInfluence:"
		print diiReligionInfluence
		
		# Choose best rules
		# TODO randomly choose one of the best rules, not the first
		maxPoints = 0
		pCurrentRule = None
		
		for pStateRule in self.lpAvailableStateRules :
			print "HOMELAND: Found rule in Province %s: %s"%( sProvince, pStateRule.sName )
			iCurrentPoints = pStateRule.getPoints( sProvince, iEra, diiReligionInfluence, iStateReligion )
			print "HOMELAND: StateRule %s has %d points"%( pStateRule.sName, iCurrentPoints )
			if iCurrentPoints > maxPoints :
				#check if that or a similar StateRule is used
				# TODO: What if Civ is already used?
				bProvinceUnused = True
				for iLoopPlayer, pRule in dipRulesForPlayer.iteritems() :
					if not ( pRule == None or iPlayer == iLoopPlayer ):
						if( gc.getPlayer( iLoopPlayer ).isAlive() and ( pStateRule == pRule or pStateRule.sDesc == pRule.sDesc or pStateRule.sShortDesc == pRule.sShortDesc or pStateRule.sAdjective == pRule.sAdjective  ) ):
							bProvinceUnused = False
							break
				if( bProvinceUnused ) :
					maxPoints = iCurrentPoints
					pCurrentRule = pStateRule
		return pCurrentRule

### globals
dslsStateModules = {}
#Europe
dslsStateModules["Europe"] = StateModule(["C:/Programme/2K Games/Firaxis Games/Sid Meier's Civilization 4 Complete/Beyond the Sword/Mods/LoR Homeland CIVS+/Assets/Python/Homeland/States/europeStates.xml"])
#dslsStateModules["Europe"] = StateModule(["europeStates.xml"])

def getCiv( sStateModule, sProvince, lpCities ):
	'returns the type string of the civ'
	pStateRule = dslsStateModules[sStateModule].getStateRuleForCities( sProvince, lpCities )
	if( pStateRule == None ) :
		print "HOMELAND: No rule chosen in %s"%( sProvince )
		return -1
	else :
		print "HOMELAND: Rule chosen in %s: %s"%( sProvince, pStateRule.sName )
		return pStateRule.getCiv()

def getPlayerName( sStateModule, sProvince, iPlayer ):
	'returns [desc, shortDesc, adjective]'
	print "Asking for player name in Province %s"%( sProvince )
	
	pStateRule = dslsStateModules[sStateModule].getStateRuleForPlayer( sProvince, iPlayer )
	if( pStateRule == None or pStateRule.sDesc == None or pStateRule.sShortDesc == None or pStateRule.sAdjective == None ) :
		CyInterface().addImmediateMessage( "Player name not set for Province %s"%( sProvince ), "" )
		return None
	else :
		CyInterface().addImmediateMessage( "Player name for Province %s: %s"%( sProvince, pStateRule.sShortDesc ), "" )
		return [pStateRule.sDesc, pStateRule.sShortDesc, pStateRule.sAdjective]
